-- Locals
local PlayerName = UnitName("player")
local PlayerClass = select(2, UnitClass("player"))
local CurrentRollValue = 0
local Rolls = {}
local Players = {}
local isAdmin = false
local SilentMode = false
local ChatGrabChannel
local GameInProgress = false
local AcceptingRolls = false
local ReportChannel = ""

-- Globals

-- Libraries
local random = random
local gsub = gsub
local type = type
local floor = floor
local format = format
local strmatch = string.match
local tonumber = tonumber
local tostring = tostring
local reverse = string.reverse
local tinsert = tinsert
local tremove = tremove
local tsort = table.sort
local split = strsplit

-- Functions
local IsInRaid = IsInRaid
local IsInGroup = IsInGroup
local IsInGuild = IsInGuild
local LE_PARTY_CATEGORY_HOME = LE_PARTY_CATEGORY_HOME
local LE_PARTY_CATEGORY_INSTANCE = LE_PARTY_CATEGORY_INSTANCE
local SendAddonMessage = SendAddonMessage

if IsoGamble_Stats == nil then
    IsoGamble_Stats = {};
end

local Comma = function(number)
	if (not number) then
		return
	end

	if (type(number) ~= "number") then
		number = tonumber(number)
	end

	local Number = format("%.0f", floor(number + 0.5))
   	local Left, Number, Right = strmatch(Number, "^([^%d]*%d)(%d+)(.-)$")

	return Left and Left .. reverse(gsub(reverse(Number), "(%d%d%d)", "%1,")) or number
end

-- Running A Roll

local GetRoll = function()
	local Roll = random(1, CurrentRollValue)

	return Roll
end

-- Sorting Rolls, Recording Stats, and Reporting Results

local SortRolls = function()
	tsort(Rolls, function(a, b)
		return a[2] > b[2]
	end)

	local Winner = Rolls[1]
	local Loser = Rolls[#Rolls]
	local Diff = Winner[2] - Loser[2]

	if (not IsoGamble_Stats[Winner[1]]) then
		IsoGamble_Stats[Winner[1]] = 0
	end

	if (not IsoGamble_Stats[Loser[1]]) then
		IsoGamble_Stats[Loser[1]] = 0
	end

	IsoGamble_Stats[Winner[1]] = IsoGamble_Stats[Winner[1]] + Diff
	IsoGamble_Stats[Loser[1]] = IsoGamble_Stats[Loser[1]] - Diff

	IsoGamble.BottomLabel:SetText(format("%s owes %s %sg", Loser[1], Winner[1], Comma(Diff)))

	-- If the user is set as an admin (meaning they started the game), and if silent mode is disabled, they will report the results to the current channel.
	if(isAdmin) then
		if(not SilentMode) then
			SendChatMessage(format("[IG] %s owes %s %s gold!", Loser[1], Winner [1], Comma(Diff)), ReportChannel, nil)
		end
		--isAdmin = false
		--print("DEBUG | [End Game] Removed your admin status")
	end

	for i = #Rolls, 1, -1 do
		tremove(Rolls, 1)
	end
end

-- Checking Rolls

local CheckRolls = function()
	local NumPlayers = #Players
	local Count = 0

	for i = 1, #Players do
		if Players[i].HasRolled then
			Count = Count + 1
		end
	end

	if (Count == NumPlayers) then
		SortRolls()
	end
end

-- Posting Stats

local PostStats = function()
	local Temp = {}
	local i = 1

	for k, v in pairs(IsoGamble_Stats) do
		Temp[i] = {k, v}
		i = i + 1
	end

	if (i == 1) then
		print("No stats yet!")

		return
	end

	tsort(Temp, function(a, b)
		return a[2] > b[2]
	end)

	for i = 1, #Temp do
		print(format("%d. %s: %s", i, Temp[i][1], Comma(Temp[i][2])))
	end

	Temp = nil
end

-- GUI
local Blank = "Interface\\AddOns\\IsoGamble\\Blank.tga"
local Font = "Interface\\AddOns\\IsoGamble\\PTSans.ttf"
local FontColor = {220/255, 220/255, 220/255}

local Backdrop = {
	bgFile = Blank,
	edgeFile = Blank,
	tile = false, tileSize = 0, edgeSize = 1,
	insets = {left = 1, right = 1, top = 1, bottom = 1},
}

local BackdropBorder = {
	edgeFile = Blank,
	edgeSize = 1,
	insets = {left = 0, right = 0, top = 0, bottom = 0},
}

local SetTemplate = function(self)
	self:SetBackdrop(Backdrop)
	self:SetBackdropBorderColor(0, 0, 0)
	self:SetBackdropColor(0.21, 0.21, 0.21)
end

local SetTemplateDark = function(self)
	self:SetBackdrop(Backdrop)
	self:SetBackdropBorderColor(0, 0, 0)
	self:SetBackdropColor(0.12, 0.12, 0.12)
end

local SendEvent = function(event, arg1)
	local Channel

-- ADD: Conditional variables to each that allow the current chat channel to be shown in the admin panel.
	if IsInRaid() then
		Channel = (not IsInRaid(LE_PARTY_CATEGORY_HOME) and IsInRaid(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "RAID"
		ReportChannel = "RAID"
	elseif IsInGroup() then
		Channel = (not IsInGroup(LE_PARTY_CATEGORY_HOME) and IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "PARTY"
		ReportChannel = "PARTY"
	elseif IsInGuild() then
		Channel = "GUILD"
		ReportChannel = "GUILD"
	end

	if Channel then
		local Event = event .. ":" .. tostring(arg1)

		SendAddonMessage("IsoGamble", Event, Channel)
	end
end

local GUI = CreateFrame("Frame", "IsoGamble", UIParent)
GUI:SetSize(228, 160) -- Default is 228, 160
GUI:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
SetTemplate(GUI)
GUI:SetMovable(true)
GUI:EnableMouse(true)
GUI:SetUserPlaced(true)
GUI:RegisterForDrag("LeftButton")
GUI:SetScript("OnDragStart", GUI.StartMoving)
GUI:SetScript("OnDragStop", GUI.StopMovingOrSizing)
GUI:Hide()

local Top = CreateFrame("Frame", nil, GUI)
Top:SetSize(228, 21) -- Default is 228, 21
Top:SetPoint("BOTTOM", GUI, "TOP", 0, -1)
SetTemplateDark(Top)

GUI.TopLabel = Top:CreateFontString(nil, "OVERLAY")
GUI.TopLabel:SetPoint("TOPLEFT", Top, "TOPLEFT", 4, -4)
GUI.TopLabel:SetFont(Font, 16)
GUI.TopLabel:SetTextColor(unpack(FontColor))
GUI.TopLabel:SetShadowOffset(1.25, -1.25)
GUI.TopLabel:SetShadowColor(0, 0, 0)

-- Exit Button

local Close = CreateFrame("Button", nil, Top)
Close:SetSize(21, 21)
Close:SetPoint("TOPRIGHT", Top, "TOPRIGHT", 0, 0)
Close:SetFrameStrata("MEDIUM")
SetTemplateDark(Close)
Close:SetScript("OnMouseUp", function(self)
	GUI:Hide()
end)
Close:SetScript("OnEnter", function(self) self.X:SetTextColor(1, 0.1, 0.1) end)
Close:SetScript("OnLeave", function(self) self.X:SetTextColor(unpack(FontColor)) end)

Close.X = Close:CreateFontString(nil, "OVERLAY")
Close.X:SetPoint("CENTER", Close, "CENTER", 1, -1)
Close.X:SetFont(Font, 16)
Close.X:SetTextColor(unpack(FontColor))
Close.X:SetText("×")
Close.X:SetShadowOffset(1.25, -1.25)
Close.X:SetShadowColor(0, 0, 0)

-- Admin panel
local Admin = CreateFrame("Frame", "IsoGambleAdmin", GUI)
Admin:SetSize(112, 96) --Original Size: 120, 96
Admin:SetPoint("BOTTOMLEFT", GUI, "TOPLEFT", 0, 23)
SetTemplate(Admin)
Admin:Hide()

local AdminTop = CreateFrame("Frame", nil, Admin)
AdminTop:SetSize(Admin:GetSize(), 21)
AdminTop:SetPoint("BOTTOM", Admin, "TOP", 0, -1)
SetTemplateDark(AdminTop)

AdminTop.TopLabel = AdminTop:CreateFontString(nil, "OVERLAY")
AdminTop.TopLabel:SetPoint("CENTER", AdminTop, "CENTER", 0, -1) -- Original Point: AdminTop.TopLabel:SetPoint("TOPLEFT", AdminTop, "TOPLEFT", 4, -4)
AdminTop.TopLabel:SetFont(Font, 16)
AdminTop.TopLabel:SetTextColor(unpack(FontColor))
AdminTop.TopLabel:SetText("Admin Panel")
AdminTop.TopLabel:SetShadowOffset(1.25, -1.25)
AdminTop.TopLabel:SetShadowColor(0, 0, 0)

-- Commands panel
local SettingsPanel = CreateFrame("Frame", "IsoGambleSettingsPanel", GUI)
SettingsPanel:SetSize(112, 96) --Original Size: 120, 96
SettingsPanel:SetPoint("BOTTOMRIGHT", GUI, "TOPRIGHT", 0, 23)
SetTemplate(SettingsPanel)
SettingsPanel:Hide()

local SettingsTop = CreateFrame("Frame", nil, SettingsPanel)
SettingsTop:SetSize(SettingsPanel:GetSize(), 21)
SettingsTop:SetPoint("BOTTOM", SettingsPanel, "TOP", 0, -1)
SetTemplateDark(SettingsTop)

SettingsTop.TopLabel = SettingsTop:CreateFontString(nil, "OVERLAY")
SettingsTop.TopLabel:SetPoint("CENTER", SettingsTop, "CENTER", 0, -1) -- Original Point: AdminTop.TopLabel:SetPoint("TOPLEFT", AdminTop, "TOPLEFT", 4, -4)
SettingsTop.TopLabel:SetFont(Font, 16)
SettingsTop.TopLabel:SetTextColor(unpack(FontColor))
SettingsTop.TopLabel:SetText("Settings")
SettingsTop.TopLabel:SetShadowOffset(1.25, -1.25)
SettingsTop.TopLabel:SetShadowColor(0, 0, 0)

-----------------------
--    USER BUTTONS   --
-----------------------

-- Button Enter and Button Leave Functions
local ButtonOnEnter = function(self)
	self:SetBackdropColor(0.17, 0.17, 0.17)
	SetTemplateDark(GameTooltip)
	GameTooltip:SetOwner(AdminTop, "ANCHOR_BOTTOMLEFT", -2, 21)
end

local ButtonOnLeave = function(self)
	self:SetBackdropColor(0.12, 0.12, 0.12)
	GameTooltip:Hide()
end

-- Enter Button
EnterButton = CreateFrame("Frame", "IsoGambleJoinButton", GUI)
EnterButton:SetSize(70, 21)
EnterButton:SetPoint("LEFT", Top, 0, 0)
SetTemplateDark(EnterButton)
EnterButton:SetScript("OnEnter", ButtonOnEnter)
EnterButton:SetScript("OnLeave", ButtonOnLeave)
EnterButton:SetScript("OnMouseUp", function(self)
	SendEvent("ADD_PLAYER", PlayerName)
end)

EnterButton.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

EnterButton.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

EnterButton.Label = EnterButton:CreateFontString(nil, "OVERLAY")
EnterButton.Label:SetPoint("CENTER", EnterButton, 0, 0)
EnterButton.Label:SetFont(Font, 14)
EnterButton.Label:SetJustifyH("CENTER")
EnterButton.Label:SetTextColor(unpack(FontColor))
EnterButton.Label:SetText("Join")
EnterButton.Label:SetShadowOffset(1.25, -1.25)
EnterButton.Label:SetShadowColor(0, 0, 0)

-- Pass Button
PassButton = CreateFrame("Frame", "IsoGamblePassButton", GUI)
PassButton:SetSize(70, 21)
PassButton:SetPoint("LEFT", EnterButton, "RIGHT", -1, 0)
SetTemplateDark(PassButton)
PassButton:SetScript("OnEnter", ButtonOnEnter)
PassButton:SetScript("OnLeave", ButtonOnLeave)
PassButton:SetScript("OnMouseUp", function(self)
	self:Disable()
	SendEvent("REMOVE_PLAYER", PlayerName)
	IsoGambleJoinButton:Disable()
end)

PassButton.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

PassButton.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

PassButton.Label = PassButton:CreateFontString(nil, "OVERLAY")
PassButton.Label:SetPoint("CENTER", PassButton, 0, 0)
PassButton.Label:SetFont(Font, 14)
PassButton.Label:SetJustifyH("CENTER")
PassButton.Label:SetTextColor(unpack(FontColor))
PassButton.Label:SetText("Pass")
PassButton.Label:SetShadowOffset(1.25, -1.25)
PassButton.Label:SetShadowColor(0, 0, 0)

-- Roll button
RollButton = CreateFrame("Frame", "IsoGambleRollButton", GUI)
RollButton:SetSize(70, 21)
RollButton:SetPoint("LEFT", PassButton, "RIGHT", -1, 0)
SetTemplateDark(RollButton)
RollButton:SetScript("OnEnter", ButtonOnEnter)
RollButton:SetScript("OnLeave", ButtonOnLeave)
RollButton:SetScript("OnMouseUp", function(self)
	local Roll = GetRoll()

	SendEvent("PLAYER_ROLL", PlayerName..":"..tostring(Roll))

	--[[for i = 1, #Players do
		if (Players[i].Name == PlayerName and not Players[i].HasRolled) then
			Players[i].HasRolled = true
			Players[i].Total:SetText(Comma(Roll))

			SendEvent("PLAYER_ROLL", PlayerName..":"..tostring(Roll))
		end
	end]]
end)

RollButton.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

RollButton.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

RollButton.Label = RollButton:CreateFontString(nil, "OVERLAY")
RollButton.Label:SetPoint("CENTER", RollButton, 0, 0)
RollButton.Label:SetFont(Font, 14)
RollButton.Label:SetJustifyH("CENTER")
RollButton.Label:SetTextColor(unpack(FontColor))
RollButton.Label:SetText("Roll!")
RollButton.Label:SetShadowOffset(1.25, -1.25)
RollButton.Label:SetShadowColor(0, 0, 0)

local Bottom = CreateFrame("Frame", nil, GUI)
Bottom:SetSize(228, 21) -- Default is 228
Bottom:SetPoint("TOP", GUI, "BOTTOM", 0, 1)
SetTemplateDark(Bottom)

GUI.BottomLabel = Bottom:CreateFontString(nil, "OVERLAY")
GUI.BottomLabel:SetPoint("LEFT", Bottom, 22, 0)
GUI.BottomLabel:SetFont(Font, 12)
GUI.BottomLabel:SetTextColor(unpack(FontColor))
GUI.BottomLabel:SetJustifyH("LEFT")
GUI.BottomLabel:SetShadowOffset(1.25, -1.25)
GUI.BottomLabel:SetShadowColor(0, 0, 0)

local PlayerRoll = function(player)
	for i = 1, #Players do
		if (Players[i].Name == player and not Players[i].HasRolled) then
			Players[i].HasRolled = true
			local Roll = GetRoll()

			tinsert(Rolls, {Players[i].Name, Roll})

			Players[i].Total:SetText(Comma(Roll))
			-- Send the roll value to other clients.

			CheckRolls()

		end
	end
end

local SortPlayers = function()
	for i = 1, #Players do
		if (i == 1) then
			Players[i]:SetPoint("TOPLEFT", GUI, "TOPLEFT", 3, -3)
		else
			Players[i]:SetPoint("TOP", Players[i-1], "BOTTOM", 0, -2)
		end
	end
end

local ChatAddPlayer = function(chatName)
	local charname, realmname = strsplit("-", chatName)
--local insname = strlower(charname)
	--print("DEBUG | REGISTERED ADD NAME: ", charname)
	SendEvent("ADD_PLAYER", charname)
end

local ChatRemovePlayer = function(chatName)
	local charname, realmname = strsplit("-", chatName)
--local insname = strlower(charname)
	--print("DEBUG | REGISTERED REMOVE NAME: ", charname)
	SendEvent("REMOVE_PLAYER", charname)
end

local AddPlayer = function(name)
	for i = 1, #Players do
		if (Players[i].Name == name) then
			return
		end
	end

	local PlayerSlot = CreateFrame("Frame", nil, GUI)
	PlayerSlot:SetSize(222, 21)
	SetTemplateDark(PlayerSlot)

	PlayerSlot.Name = name

	PlayerSlot.Label = PlayerSlot:CreateFontString(nil, "OVERLAY")
	PlayerSlot.Label:SetPoint("LEFT", PlayerSlot, 4, 0)
	PlayerSlot.Label:SetFont(Font, 14)
	PlayerSlot.Label:SetJustifyH("LEFT")
	PlayerSlot.Label:SetTextColor(unpack(FontColor))
	PlayerSlot.Label:SetText(name)
	PlayerSlot.Label:SetShadowOffset(1.25, -1.25)
	PlayerSlot.Label:SetShadowColor(0, 0, 0)

	PlayerSlot.TotalFrame = CreateFrame("Frame", nil, PlayerSlot)
	PlayerSlot.TotalFrame:SetSize(80, 21)
	PlayerSlot.TotalFrame:SetPoint("RIGHT", PlayerSlot, 0, 0)
	SetTemplateDark(PlayerSlot.TotalFrame)

	PlayerSlot.Total = PlayerSlot.TotalFrame:CreateFontString(nil, "OVERLAY")
	PlayerSlot.Total:SetPoint("LEFT", PlayerSlot.TotalFrame, 4, 0)
	PlayerSlot.Total:SetFont(Font, 14)
	PlayerSlot.Total:SetJustifyH("LEFT")
	PlayerSlot.Total:SetTextColor(unpack(FontColor))
	PlayerSlot.Total:SetShadowOffset(1.25, -1.25)
	PlayerSlot.Total:SetShadowColor(0, 0, 0)

	tinsert(Players, PlayerSlot)

	SortPlayers()
end

local RemovePlayer = function(name)
	local Player

	for i = 1, #Players do
		if (Players[i].Name == name) then
			Player = Players[i]
			tremove(Players, i)
			break
		end
	end

	if Player then
		Player:Hide()
		SortPlayers()
	end
end

-- Editbox
local EditBoxOnMouseDown = function(self)
	self:SetAutoFocus(true)
	self:SetText("") -- Clears the editbox whenever it is clicked on.
end

local EditBoxOnEditFocusLost = function(self)
	self:SetAutoFocus(false)
end

local EditBoxOnEscapePressed = function(self)
	self:SetAutoFocus(false)
	self:ClearFocus()
end

local EditBoxOnEnterPressed = function(self)
	self:SetAutoFocus(false)
	self:ClearFocus()

	local Value = self:GetText()

	if (Value == "" or Value == " ") then
		self:SetText(CurrentRollValue)

		return
	end
end

------------------------
--    ADMIN BUTTONS   --
------------------------

local EditBox = CreateFrame("Frame", nil, Admin)
EditBox:SetPoint("TOPLEFT", Admin, 3, -3)
EditBox:SetSize(Admin:GetSize()-6, 21)
SetTemplateDark(EditBox)
EditBox:EnableMouse(true)

EditBox.Box = CreateFrame("EditBox", nil, EditBox)
EditBox.Box:SetPoint("TOPLEFT", EditBox, 4, -4)
EditBox.Box:SetPoint("BOTTOMRIGHT", EditBox, -4, 2)
EditBox.Box:SetFont(Font, 16)
EditBox.Box:SetText("Enter Wager...")
EditBox.Box:SetShadowColor(0, 0, 0)
EditBox.Box:SetShadowOffset(1.25, -1.25)
EditBox.Box:SetMaxLetters(6)
EditBox.Box:SetAutoFocus(false)
EditBox.Box:SetNumeric(true)
EditBox.Box:EnableKeyboard(true)
EditBox.Box:EnableMouse(true)
EditBox.Box:SetScript("OnMouseDown", EditBoxOnMouseDown)
EditBox.Box:SetScript("OnEscapePressed", EditBoxOnEscapePressed)
EditBox.Box:SetScript("OnEnterPressed", EditBoxOnEnterPressed)
EditBox.Box:SetScript("OnEditFocusLost", EditBoxOnEditFocusLost)

-- New Game Button

AcceptRolls = CreateFrame("Frame", nil, Admin)
AcceptRolls:SetSize(Admin:GetSize()-6, 21)
AcceptRolls:SetPoint("TOPLEFT", EditBox, "BOTTOMLEFT", 0, -2)
SetTemplateDark(AcceptRolls)
AcceptRolls:SetScript("OnEnter", ButtonOnEnter)
AcceptRolls:HookScript("OnEnter", function(self)
	GameTooltip:SetText("New Game")
	GameTooltip:AddLine("Step 1. Starts a new game.", 1, 1, 1, true)
	GameTooltip:Show()
end)
AcceptRolls:SetScript("OnLeave", ButtonOnLeave)
AcceptRolls:SetScript("OnMouseUp", function(self)
	if (EditBox.Box:GetText() ~= "Enter Wager") then
		SendEvent("SET_ROLL", EditBox.Box:GetText())
		SendEvent("START_GAME")
		isAdmin = true
		GameInProgress = true
		AcceptingRolls = true
		--print("DEBUG | Added your admin status.")
		--print("DEBUG | Channel: ", Channel, " / ReportChannel: ", ReportChannel)
		-- Tells us that the user is the admin because they started the game.

	else
		print("[IG] Please enter a wager before starting a new game!")
	end
end)

AcceptRolls.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

AcceptRolls.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

AcceptRolls.Label = AcceptRolls:CreateFontString(nil, "OVERLAY")
AcceptRolls.Label:SetPoint("CENTER", AcceptRolls, 0, 0)
AcceptRolls.Label:SetFont(Font, 14)
AcceptRolls.Label:SetJustifyH("CENTER")
AcceptRolls.Label:SetTextColor(unpack(FontColor))
AcceptRolls.Label:SetText("New Game")
AcceptRolls.Label:SetShadowOffset(1.25, -1.25)
AcceptRolls.Label:SetShadowColor(0, 0, 0)

-- Close
CloseGame = CreateFrame("Frame", nil, Admin)
CloseGame:SetSize(Admin:GetSize()-6, 21)
CloseGame:SetPoint("TOPLEFT", AcceptRolls, "BOTTOMLEFT", 0, -2)
SetTemplateDark(CloseGame)
CloseGame:SetScript("OnEnter", ButtonOnEnter)
CloseGame:HookScript("OnEnter", function(self)
	GameTooltip:SetText("Start Rolling")
	GameTooltip:AddLine("Step 2. Closes entry to the game and allows users to roll.", 1, 1, 1, true)
	GameTooltip:Show()
end)
CloseGame:SetScript("OnLeave", ButtonOnLeave)
CloseGame:SetScript("OnMouseUp", function(self)
	SendEvent("CLOSE_GAME")
end)

CloseGame.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

CloseGame.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

CloseGame.Label = CloseGame:CreateFontString(nil, "OVERLAY")
CloseGame.Label:SetPoint("CENTER", CloseGame, 0, 0)
CloseGame.Label:SetFont(Font, 14)
CloseGame.Label:SetJustifyH("CENTER")
CloseGame.Label:SetTextColor(unpack(FontColor))
CloseGame.Label:SetText("Start Rolling")
CloseGame.Label:SetShadowOffset(1.25, -1.25)
CloseGame.Label:SetShadowColor(0, 0, 0)

-- Reset
Reset = CreateFrame("Frame", nil, Admin)
Reset:SetSize(Admin:GetSize()-6, 21)
Reset:SetPoint("TOPLEFT", CloseGame, "BOTTOMLEFT", 0, -2)
SetTemplateDark(Reset)
Reset:SetScript("OnEnter", ButtonOnEnter)
Reset:HookScript("OnEnter", function(self)
	GameTooltip:SetText("Reset Game")
	GameTooltip:AddLine("Cancels the currently active game.", 1, 1, 1, true)
	GameTooltip:Show()
end)
Reset:SetScript("OnLeave", ButtonOnLeave)
Reset:SetScript("OnMouseUp", function(self)
	if(isAdmin == true) then
		SendEvent("RESET_ALL")
	end
	if(SilentMode == false and isAdmin == true) then
		SendChatMessage("[IG] Game has been cancelled.", ReportChannel, nil)
	end
end)

Reset.Label = Reset:CreateFontString(nil, "OVERLAY")
Reset.Label:SetPoint("CENTER", Reset, 0, 0)
Reset.Label:SetFont(Font, 14)
Reset.Label:SetJustifyH("CENTER")
Reset.Label:SetTextColor(unpack(FontColor))
Reset.Label:SetText("Reset Game")
Reset.Label:SetShadowOffset(1.25, -1.25)
Reset.Label:SetShadowColor(0, 0, 0)

CloseGame:Disable()
EnterButton:Disable()
PassButton:Disable()
RollButton:Disable()

---------------------------
--    SETTINGS BUTTONS   --
---------------------------

-- Silent Button
SilentButton = CreateFrame("Frame", nil, SettingsPanel)
SilentButton:SetSize(SettingsPanel:GetSize()-6, 21)
SilentButton:SetPoint("TOPLEFT", SettingsPanel, 3, -3)
SetTemplateDark(SilentButton)
SilentButton:SetScript("OnEnter", ButtonOnEnter)
SilentButton:HookScript("OnEnter", function(self)
	GameTooltip:SetText("Silent Mode")
	if(SilentMode == true) then
		GameTooltip:AddLine("Currently On: Players must have the addon to participate.", 1, 1, 1, true)
	else
		GameTooltip:AddLine("Currently Off: Players who do not have the addon may participate via chat.", 1, 1, 1, true)
	end
	GameTooltip:Show()
end)
SilentButton:SetScript("OnLeave", ButtonOnLeave)
SilentButton:SetScript("OnMouseUp", function(self)
	if (SilentMode == false) then
		SilentMode = true
		self.Label:SetText("Silent Mode (ON)")
		--print("DEBUG | Turned ON silent mode.")
	else
		SilentMode = false
		self.Label:SetText("Silent Mode (OFF)")
		--print("DEBUG | Turned OFF silent mode.")
	end
end)

SilentButton.Disable = function(self)
	self.Label:SetTextColor(0.3, 0.3, 0.3)
	self:EnableMouse(false)
end

SilentButton.Enable = function(self)
	self.Label:SetTextColor(unpack(FontColor))
	self:EnableMouse(true)
end

SilentButton.Label = SilentButton:CreateFontString(nil, "OVERLAY")
SilentButton.Label:SetPoint("CENTER", SilentButton, 0, 0)
SilentButton.Label:SetFont(Font, 14)
SilentButton.Label:SetJustifyH("CENTER")
SilentButton.Label:SetTextColor(unpack(FontColor))
SilentButton.Label:SetText("Silent Mode (OFF)")
SilentButton.Label:SetShadowOffset(1.25, -1.25)
SilentButton.Label:SetShadowColor(0, 0, 0)

-- Chat window
local ChatFrame = CreateFrame("Frame", nil, GUI)
ChatFrame:SetPoint("TOPLEFT", Top, "TOPRIGHT", 2, 0)
ChatFrame:SetSize(260, 180)
SetTemplate(ChatFrame)
ChatFrame:Hide()

ChatFrame.Chat = CreateFrame("ScrollingMessageFrame", nil, ChatFrame)
ChatFrame.Chat:SetPoint("CENTER", ChatFrame, 2, 3)
ChatFrame.Chat:SetSize(ChatFrame:GetWidth() - 8, ChatFrame:GetHeight() - 6)
ChatFrame.Chat:SetFont(Font, 14)
ChatFrame.Chat:SetShadowColor(0, 0, 0)
ChatFrame.Chat:SetShadowOffset(1.25, -1.25)
ChatFrame.Chat:SetFading(false)
ChatFrame.Chat:SetJustifyH("LEFT")
ChatFrame.Chat:SetMaxLines(50)
ChatFrame.Chat:SetScript("OnMouseWheel", function(self, delta)
	if (delta == 1) then
		self:ScrollUp()
	else
		self:ScrollDown()
	end
end)

-- Editbox
local EditBoxOnMouseDown = function(self)
	self:SetText("")
	self:SetAutoFocus(true)
end

local EditBoxOnEditFocusLost = function(self)
	self:SetAutoFocus(false)
end

local EditBoxOnEscapePressed = function(self)
	self:SetText("")

	self:SetAutoFocus(false)
	self:ClearFocus()

	self:SetText("|cffB0B0B0Chat...|r")
end

local EditBoxOnEnterPressed = function(self)
	self:SetAutoFocus(false)
	self:ClearFocus()

	local Value = self:GetText()

	if (Value == "" or Value == " ") then
		self:SetText("|cffB0B0B0Chat...|r")

		return
	end

	SendEvent(format("CHAT_MSG:%s:%s:%s", PlayerName, PlayerClass, Value))

	self:SetText("|cffB0B0B0Chat...|r")
end

local EditBox = CreateFrame("Frame", nil, ChatFrame)
EditBox:SetPoint("TOPLEFT", ChatFrame, "BOTTOMLEFT", 0, 1)
EditBox:SetSize(260, 21)
SetTemplateDark(EditBox)
EditBox:EnableMouse(true)

EditBox.Box = CreateFrame("EditBox", nil, EditBox)
EditBox.Box:SetPoint("TOPLEFT", EditBox, 5, -1)
EditBox.Box:SetPoint("BOTTOMRIGHT", EditBox, -5, 1)
EditBox.Box:SetFont(Font, 14)
EditBox.Box:SetText("|cffB0B0B0Chat...|r")
EditBox.Box:SetShadowColor(0, 0, 0)
EditBox.Box:SetShadowOffset(1.25, -1.25)
EditBox.Box:SetMaxLetters(255)
EditBox.Box:SetAutoFocus(false)
EditBox.Box:EnableKeyboard(true)
EditBox.Box:EnableMouse(true)
EditBox.Box:SetScript("OnMouseDown", EditBoxOnMouseDown)
EditBox.Box:SetScript("OnEscapePressed", EditBoxOnEscapePressed)
EditBox.Box:SetScript("OnEnterPressed", EditBoxOnEnterPressed)
EditBox.Box:SetScript("OnEditFocusLost", EditBoxOnEditFocusLost)

-- Chat toggle
local ChatToggle = CreateFrame("Button", nil, GUI)
ChatToggle:SetSize(21, 21)
ChatToggle:SetPoint("BOTTOMRIGHT", Bottom, "BOTTOMRIGHT", 0, 0)
ChatToggle:SetFrameStrata("MEDIUM")
SetTemplateDark(ChatToggle)
ChatToggle:SetScript("OnMouseUp", function(self)
	if self.NeedsReset then
		self.Arrow:SetTextColor(1, 1, 1)
		self.NeedsReset = false
	end

	if ChatFrame:IsShown() then
		ChatFrame:Hide()
		self.Arrow:SetText("►")
	else
		ChatFrame:Show()
		self.Arrow:SetText("◄")
	end
end)

ChatToggle.Arrow = ChatToggle:CreateFontString(nil, "OVERLAY")
ChatToggle.Arrow:SetPoint("CENTER", ChatToggle, "CENTER", 0, 0)
ChatToggle.Arrow:SetFont("Interface\\AddOns\\IsoGamble\\Arial.ttf", 12)
ChatToggle.Arrow:SetTextColor(unpack(FontColor))
ChatToggle.Arrow:SetText("►")
ChatToggle.Arrow:SetShadowOffset(1.25, -1.25)
ChatToggle.Arrow:SetShadowColor(0, 0, 0)

-- Admin Toggle
local AdminToggle = CreateFrame("Button", nil, GUI)
AdminToggle:SetSize(21, 21)
AdminToggle:SetPoint("BOTTOMLEFT", Bottom, "BOTTOMLEFT", 0, 0)
AdminToggle:SetFrameStrata("MEDIUM")
SetTemplateDark(AdminToggle)
AdminToggle:SetScript("OnMouseUp", function(self)
	if Admin:IsShown() then
		Admin:Hide()
		SettingsPanel:Hide()
		self.Arrow:SetText("A")
		self.Arrow:SetTextColor(unpack(FontColor))
	else
		Admin:Show()
		SettingsPanel:Show()
		self.Arrow:SetText("A")
		self.Arrow:SetTextColor(0,1,0,1)
	end
end)

AdminToggle.Arrow = AdminToggle:CreateFontString(nil, "OVERLAY")
AdminToggle.Arrow:SetPoint("CENTER", AdminToggle, "CENTER", 1, 0)
AdminToggle.Arrow:SetFont("Interface\\AddOns\\IsoGamble\\Arial.ttf", 12)
AdminToggle.Arrow:SetTextColor(unpack(FontColor))
AdminToggle.Arrow:SetText("A")
AdminToggle.Arrow:SetShadowOffset(1.25, -1.25)
AdminToggle.Arrow:SetShadowColor(0, 0, 0)

-- Debug Stuff. Will be cleaned up.

-- /run __SGAdd("Vexisle")
-- /run __SGAdd("Vexisle"); __SGAdd("Cobelarusu"); __SGAdd("Banktre"); __SGAdd("Obsidiana")
__SGAdd = AddPlayer
__SGRem = RemovePlayer --/run __SGRem("Banktre")
__SGRoll = PlayerRoll -- /run __SGRoll("Vexisle"); __SGRoll("Cobelarusu"); __SGRoll("Banktre"); __SGRoll("Obsidiana")

--- /run IsoGambleAdmin:Show()

-- /run __SGAdd("Vexxisle"); __SGAdd("Cobelarusu"); __SGAdd("Banktre"); __SGAdd("Obsidiana"); __SGRoll("Vexxisle"); __SGRoll("Cobelarusu"); __SGRoll("Banktre"); __SGRoll("Obsidiana")

-- /run __SGAdd("Vexisle"); __SGAdd("Cobelarusu"); __SGAdd("Banktre"); __SGAdd("Obsidiana"); __SGAdd("Bigroostie"); __SGAdd("Schwanks"); __SGRoll("Vexisle"); __SGRoll("Cobelarusu"); __SGRoll("Banktre"); __SGRoll("Obsidiana")

-- /run __DUMPSGSTATS()

local Events = {}

Events["RESET_ALL"] = function()
	for i = 1, #Players do
		Players[1].HasRolled = false
		Players[1].Total:SetText("")
		RemovePlayer(Players[1].Name)
	--Players[1].IsAdmin = false -- When anyone resets the game, it sets isAdmin for all players to false. // IGNORE FOR NOW, AS WELL AS THE ADD_ADMIN AND REMOVE_ADMIN EVENTS
	end

	isAdmin = false
	GameInProgress = false
	--print("DEBUG | [Reset] Removed your admin status")

	IsoGamble.BottomLabel:SetText("")

	AcceptRolls:Enable()
	CloseGame:Disable()
	EnterButton:Disable()
	PassButton:Disable()
	RollButton:Disable()

	SilentButton:Enable()
end

Events["ADD_PLAYER"] = function(name)
	AddPlayer(name)
end

Events["REMOVE_PLAYER"] = function(name)
	RemovePlayer(name)
end

Events["PLAYER_ROLL"] = function(name, value)
	for i = 1, #Players do
		if (Players[i].Name == name and not Players[i].HasRolled) then
			Players[i].HasRolled = true
			Players[i].Total:SetText(Comma(value))

			tinsert(Rolls, {Players[i].Name, tonumber(value)})

			CheckRolls()
		end
	end
end

Events["SET_ROLL"] = function(value)
	CurrentRollValue = tonumber(value)
	GUI.BottomLabel:SetText("Current roll is for " .. Comma(CurrentRollValue) .. ".")
end

Events["START_GAME"] = function()
	if (not IsoGamble:IsShown()) then
		-- Alert the player that a roll is happening.
		print("A roll is happening! Type /ig or /igam to watch or join.")
	end
		AcceptRolls:Disable()
		EnterButton:Enable()
		PassButton:Enable()

		SilentButton:Disable()

	if(isAdmin == true) then
		CloseGame:Enable()

		if(SilentMode == false) then
			SendChatMessage("[IG] A new game has started! Type 1 to join.", ReportChannel, nil)
			SendChatMessage(format("[IG] Wager: %s", CurrentRollValue), ReportChannel, nil)
		end
	end

end

Events["CLOSE_GAME"] = function()

	EnterButton:Disable()
	PassButton:Disable()
	RollButton:Disable()
	CloseGame:Disable()

	AcceptingRolls = false

	if(SilentMode == false and isAdmin == true) then
		SendChatMessage("[IG] Entries have closed. Roll now!", ReportChannel, nil)
		SendChatMessage(format("[IG] Type /roll %s", CurrentRollValue), ReportChannel, nil)
	end

	for i = 1, #Players do
		if (Players[i].Name == PlayerName) then
			RollButton:Enable()

			break
		end
	end
end

Events["CALL_PAYOUT"] = function(message)
	IsoGamble.BottomLabel:SetText(message)
end

function ParseChatRoll(tempString2)
	local tempString1 = tempString2
	local player, junk, actualRoll, range = strsplit(" ", tempString1)

	function CheckPlayers(player)
		for i=1, #Players do
			if (Players[i].Name == tostring(player)) then
				--print(Players[i].Name)
				return true
			end
		end
		return false
	end

	--print ("DEBUG | CheckPlayers Result: ", CheckPlayers(player))

	if junk == "rolls" and CheckPlayers(player)==true then
		minRoll, maxRoll = strsplit("-",range)
		minRoll = tonumber(strsub(minRoll,2))
		maxRoll = tonumber(strsub(maxRoll,1,-2))
		actualRoll = tonumber(actualRoll)

		--print("DEBUG | minRoll:", minRoll, " maxRoll: ", maxRoll)

		if(minRoll == 1 and maxRoll == CurrentRollValue) then
			SendEvent("PLAYER_ROLL", player..":"..tostring(actualRoll))
		end

	end
end

local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("CHAT_MSG_ADDON")
EventFrame:SetScript("OnEvent", function(self, event, prefix, message, channel, sender)
	if (prefix ~= "IsoGamble") then -- We don't need our own events either... do we?
		return
	end

	local Event, Arg1, Arg2 = split(":", message)

	if Events[Event] then
		Events[Event](Arg1, Arg2)
	elseif (Event == "CHAT_MSG") then
		if (not ChatFrame:IsShown()) then
			-- Alert that there's chatting happening.
			ChatToggle.Arrow:SetTextColor(1, 1, 0)
			ChatToggle.NeedsReset = true
		end

		local Player, Class, Message = strmatch(message, "CHAT_MSG:(%a+):(%a+):(.*):nil")
		local Hex = "|c" .. RAID_CLASS_COLORS[Class].colorStr

		ChatFrame.Chat:AddMessage(format("[%s%s|r]: %s", Hex, Player, Message))
	end
end)

local EventChatPlayer = CreateFrame("Frame")
EventChatPlayer:RegisterEvent("CHAT_MSG_PARTY")
EventChatPlayer:RegisterEvent("CHAT_MSG_PARTY_LEADER")
EventChatPlayer:RegisterEvent("CHAT_MSG_RAID")
EventChatPlayer:RegisterEvent("CHAT_MSG_RAID_LEADER")
EventChatPlayer:RegisterEvent("CHAT_MSG_GUILD")

EventChatPlayer:RegisterEvent("CHAT_MSG_SYSTEM")

EventChatPlayer:SetScript("OnEvent", function(self, event, message, sender)
	if(ReportChannel == "PARTY" and (event == "CHAT_MSG_PARTY" or event == "CHAT_MSG_PARTY_LEADER") and GameInProgress == true and AcceptingRolls == true and SilentMode == false) then
		if(message == "1") then
			--print("DEBUG | ADD SUCCESSFUL From:", sender)
			ChatAddPlayer(sender)
		elseif(message == "-1") then
			ChatRemovePlayer(sender)
		end
	elseif(ReportChannel == "RAID" and (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") and GameInProgress == true and AcceptingRolls == true and SilentMode == false) then
		if(message == "1") then
			--print("DEBUG | ADD SUCCESSFUL From:", sender)
			ChatAddPlayer(sender)
		elseif(message == "-1") then
			ChatRemovePlayer(sender)
		end
	end

	if (event == "CHAT_MSG_SYSTEM" and GameInProgress == true and SilentMode == false) then
			local tempString1 = tostring(message)
			--print("DEBUG | ON SYSTEM MESSAGE | sender: ", sender)
			--print("DEBUG | ON SYSTEM MESSAGE | event: ", event)
			--print("DEBUG | ON SYSTEM MESSAGE | message: ", message)
			--print("DEBUG | ON SYSTEM MESSAGE | arg1: ", arg1)
			--print("DEBUG | ON SYSTEM MESSAGE | tempString1: ", tempString1)
			ParseChatRoll(tempString1)
			--print("DEBUG | ROLL SUCCESSFUL From:", sender, " || ", tempString1)
	end

end)

-- Chat Commands

SLASH_IsoGamble1 = "/ig"
SLASH_IsoGamble2 = "/igam"
SlashCmdList["IsoGamble"] = function(cmd)
	if (cmd == "admin") then
		if (not GUI:IsShown()) then
			GUI:Show()
		end
		if Admin:IsShown() then
			Admin:Hide()
		else
			Admin:Show()
		end
	elseif (cmd == "settings") then
		if (not GUI:IsShown()) then
			GUI:Show()
		end
		if SettingsPanel:IsShown() then
			SettingsPanel:Hide()
		else
			SettingsPanel:Show()
		end
	elseif (cmd == "stats") then
		PostStats()
	elseif (cmd == "fake") then
		__SGAdd("Vexxisle")
		__SGAdd("Cobelarusu")
		__SGAdd("Banktre")
		__SGAdd("Obsidiana")
		__SGRoll("Vexxisle")
		__SGRoll("Cobelarusu")
		__SGRoll("Banktre")
		__SGRoll("Obsidiana")
	else
		if GUI:IsShown() then
			GUI:Hide()
		else
			GUI:Show()
		end
	end
end

RegisterAddonMessagePrefix("IsoGamble")

--[[
	Events needed. This time make the "leader" client respond to these events as well, instead of needing different sets of events.

	RESET_ALL -- Reset all players info.
	ADD_PLAYER -- Add a new player.
	REMOVE_PLAYER -- Remove a player.
	PLAYER_ROLL -- Recieve a player's roll information.
	SET_ROLL -- Tell all clients what the roll will be for.
	START_GAME -- Rolls are now open!
	CLOSE_GAME -- Stop taking new players. Let everyone roll.
	CALL_PAYOUT -- Tell everyone what the payout was.
	CHAT_MSG -- Send a chat message.
]]
